# all the 55 small coronas (except ssssss)
pc=small_coronas()

# all the 1485 pairs (i,j) of indices of small coronas, i<j
I=flatten([[(i,j) for j in range(i+1,55)] for i in range(54)],max_level=1)

# remove the pairs of coronas both without any medium disc
I=[(i,j) for (i,j) in I if 'r' in sequence(pc[i]) or 'r' in sequence(pc[j])]
# remove the pairs of coronas both without any large disc
I=[(i,j) for (i,j) in I if '1' in sequence(pc[i]) or '1' in sequence(pc[j])]

##########################
# Hidden variable method #
##########################

# compute the equations associated with the small coronas
eqpc=[equation(i,0) for i in pc] # 6.5s


# input: indices i and j of two small coronas,  k=AA or CIF
# output: cartesian product of the entries of the solutions of the associated equations
def projections((i,j),k):
    # compute the resultants
    R=ZZ[r](eqpc[i].resultant(eqpc[j],s))
    S=ZZ[s](eqpc[i].resultant(eqpc[j],r))
    RR=flatten([f[0].roots(k,multiplicities=false) for f in factor(R) if not useless(f[0])])
    SS=flatten([f[0].roots(k,multiplicities=false) for f in factor(S) if not useless(f[0])])
    # remove surely non real roots if computation on CIF (use conjugated complex roots instead?)
    if k==CIF:
        RR=[x.real() for x in RR if x.imag().contains_zero()]
        SS=[x.real() for x in SS if x.imag().contains_zero()]
    # filter real roots in (0,1)
    RR=[x for x in RR if x>0 and x<1]
    SS=[x for x in SS if x>0 and x<1]
    # cartesian product with s<r
    return flatten([[((i,j),(rr,ss)) for rr in RR if RIF(rr).endpoints()[1]>RIF(ss).endpoints()[0] and rr>ss] for ss in SS],max_level=1)

# applies projections on the whole I, result in RS or in E if error (see below)
def candidates(I,k):
    RS=[]
    E=[] # sometimes an error happens (see dedicated section below)
    for z in I:
        print(z)
        try:
            RS+=projections(z,k)
        except:
            E.append(z)
            print(sys.exc_info())
    return (RS,E)  

# 17,52: removed (see below)
I.remove((17,52))
(RS,E)=candidates(I,AA)
# 13239 candidates in 1min 30s
E.append((17,52))
# three errors :
# (4,37) : AssertionError 
# (5,38) : RuntimeError: maximum recursion depth exceeded while calling a Python object
# (17,52) : roots(AA) endless with a factor of degree 44 of S. no pb in CIF or if casts in AA[s] before...
(RSE,E2)=candidates(E,CIF)
# 179 candidates in 1.5s

#################################
# arithmetic interval filtering #
#################################

# check that the two small coronas are compatibles with the interval arithmetic values of r and s
# check also that there is at least one medium and one large corona compatible with these values
def filter_int(RS):
    RS2=[]
    for ((i,j),(rr,ss)) in RS:
        print(((i,j),(rr,ss)))
        if not check_corona_int(pc[i],0,rr,ss):
            continue
        if not check_corona_int(pc[j],0,rr,ss):
            continue
        print('Values of r and s likely compatible with the small coronas %s and %s'%(sequence(pc[i]),sequence(pc[j])))
        if find_coronas_int(rr,ss,1)==[]:
            continue
        print('Values of r and s likely compatible with at least one medium corona')
        if find_coronas_int(rr,ss,2)==[]:
            continue
        print('Values of r and s likely compatible with at least one large corona')
        RS2.append(((i,j),(rr,ss)))
    return RS2


RS2=filter_int(RS)
# 37 remaining candidates in 38s

RSE2=filter_int(RSE)
# 0 remaining candidates in 30ms
# no more error !


##############################
# exact arithmetic filtering #
##############################

# compute all the coronas
# return a list of elements ((i,j),(rr,ss),(p,m,g)) where (p,m,g) are all the small/medium/large coronas compatible with the small corona pc[i] and the medium mc[j], i.e., the value rr and ss of r and s
def filter_exact(RS):
    RS2=[]
    dup=[]  # to avoid duplicates
    k=0
    for ((i,j),(rr,ss)) in RS:
        print(k)
        k+=1
        print(((i,j),(rr,ss)))
        p=[z for z in find_coronas_int(rr,ss,0) if check_corona_exact(z,0,rr,ss)]
        if pc[i] not in p or pc[j] not in p:
            continue
        print('%d small coronas'%len(p))
        m=[z for z in find_coronas_int(rr,ss,1) if check_corona_exact(z,1,rr,ss)]
        if m==[]:
            continue
        print('%d medium coronas'%len(m))
        g=[z for z in find_coronas_int(rr,ss,2) if check_corona_exact(z,2,rr,ss)]
        if g==[]:
            continue
        print('%d large corona'%len(g))
        RS2.append(((i,j),(rr,ss),(p,m,g)))
    return RS2

RS3=filter_exact(RS2)
# 37 remaining candidates (i.e., all) in 22s
